﻿using ProjectAircraftCarrier.Entities;
using System.Drawing;

namespace ProjectAircraftCarrier.Drawnings;

/// <summary>
/// Класс, отвечающий за прорисовку и перемещение объекта-сущности
/// </summary>
public class DrawningAircraftCarrier : DrawningWarship
{
    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>
    /// <param name="additionalColor">Дополнительный цвет</param>
    /// <param name="aircraftDeck">Признак наличия взлётно-посадочной полосы</param>
    /// <param name="controlRoom">Признак наличия рубки управления</param>
    public DrawningAircraftCarrier(int speed, double weight, Color bodyColor, Color additionalColor, bool aircraftDeck, bool controlRoom) : base(145, 57)
    {
        EntityWarship = new EntityAircraftCarrier(speed, weight, bodyColor, additionalColor, aircraftDeck, controlRoom);
    }

    public override void DrawTransport(Graphics g)
    {
        if (EntityWarship == null || EntityWarship is not EntityAircraftCarrier aircraftcarrier || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }

        Pen pen = new(Color.Black);
        Brush additionalBrush = new SolidBrush(aircraftcarrier.AdditionalColor);

        base.DrawTransport(g);

        //рубка управления
        if (aircraftcarrier.ControlRoom)
        {
            g.FillRectangle(additionalBrush, _startPosX.Value + 35, _startPosY.Value + 42, 15, 15);
            g.FillRectangle(additionalBrush, _startPosX.Value + 65, _startPosY.Value + 42, 15, 15);
            g.FillRectangle(additionalBrush, _startPosX.Value + 50, _startPosY.Value + 46, 15, 7);
            g.FillRectangle(additionalBrush, _startPosX.Value + 30, _startPosY.Value + 46, 5, 7);
            g.FillRectangle(additionalBrush, _startPosX.Value + 80, _startPosY.Value + 46, 5, 7);
            g.DrawRectangle(pen, _startPosX.Value + 35, _startPosY.Value + 42, 15, 15);
            g.DrawRectangle(pen, _startPosX.Value + 65, _startPosY.Value + 42, 15, 15);
            g.DrawRectangle(pen, _startPosX.Value + 50, _startPosY.Value + 46, 15, 7);
            g.DrawRectangle(pen, _startPosX.Value + 30, _startPosY.Value + 46, 5, 7);
            g.DrawRectangle(pen, _startPosX.Value + 80, _startPosY.Value + 46, 5, 7);
            g.DrawRectangle(pen, _startPosX.Value + 39, _startPosY.Value + 46, 7, 7);
            g.DrawRectangle(pen, _startPosX.Value + 69, _startPosY.Value + 46, 7, 7);

        }

        //палуба для взлёта самолётов
        if (aircraftcarrier.AircraftDeck)
        {
            Point[] DeckPoint =
            {
                new Point(_startPosX.Value + 5, _startPosY.Value + 25),
                new Point(_startPosX.Value + 5, _startPosY.Value + 45),
                new Point(_startPosX.Value + 50, _startPosY.Value + 20),
                new Point(_startPosX.Value + 50, _startPosY.Value),
            };
            g.FillPolygon(additionalBrush, DeckPoint);
            g.DrawPolygon(pen, DeckPoint);
            g.DrawLine(pen, _startPosX.Value + 5, _startPosY.Value + 35, _startPosX.Value + 50, _startPosY.Value + 10);
        }
    }
}
