﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using ProjectAircraftCarrier.Drawnings;
using ProjectAircraftCarrier.MovementStrategy;

namespace ProjectAircraftCarrier;

/// <summary>
/// Форма работы с объектом "Авианосец"
/// </summary>
public partial class FormAircraftCarrier : Form
{
    /// <summary>
    /// Поле-объект для прорисовки объекта
    /// </summary>
    private DrawningWarship? _drawningWarship;

    /// <summary>
    /// Стратегия перемещения
    /// </summary>
    private AbstractStrategy? _strategy;

    /// <summary>
    /// Конструктор формы
    /// </summary>
    public FormAircraftCarrier()
    {
        InitializeComponent();
        _strategy = null;
    }

    /// <summary>
    /// Метод прорисовки авианосца
    /// </summary>
    private void Draw()
    {
        if (_drawningWarship == null)
        {
            return;
        }

        Bitmap bmp = new(pictureBoxAircraftCarrier.Width, pictureBoxAircraftCarrier.Height);
        Graphics gr = Graphics.FromImage(bmp);
        _drawningWarship.DrawTransport(gr);
        pictureBoxAircraftCarrier.Image = bmp;
    }

    private void CreateObject(string type)
    {
        Random random = new();
        switch (type)
        {
            case nameof(DrawningWarship):
                _drawningWarship = new DrawningWarship(random.Next(100, 300), random.Next(1000, 3000),
                    Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)));
                break;
            case nameof(DrawningAircraftCarrier):
                _drawningWarship = new DrawningAircraftCarrier(random.Next(100, 300), random.Next(1000, 3000),
                    Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)),
                    Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)),
                    Convert.ToBoolean(random.Next(0, 2)), Convert.ToBoolean(random.Next(0, 2)));
                break;
            default:
                return;
        }

        _drawningWarship.SetPictureSize(pictureBoxAircraftCarrier.Width, pictureBoxAircraftCarrier.Height);
        _drawningWarship.SetPosition(random.Next(10, 100), random.Next(10, 100));
        _strategy = null;
        comboBoxStrategy.Enabled = true;
        Draw();
    }

    /// <summary>
    /// Обработка нажатия кнопки "Создать авианосец"
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonCreateAircraftCarrier_Click(object sender, EventArgs e) => CreateObject(nameof(DrawningAircraftCarrier));

    /// <summary>
    /// Обработка нажатия кнопки "Создать военный корабль"
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonCreateWarship_Click(object sender, EventArgs e) => CreateObject(nameof(DrawningWarship));

    /// <summary>
    /// Перемещение объекта по форме (нажатие кнопок навигации)
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonMove_Click(object sender, EventArgs e)
    {
        if (_drawningWarship == null)
        {
            return;
        }

        string name = ((Button)sender)?.Name ?? string.Empty;
        bool result = false;
        switch (name)
        {
            case "buttonUp":
                result = _drawningWarship.MoveTransport(DirectionType.Up);
                break;
            case "buttonDown":
                result = _drawningWarship.MoveTransport(DirectionType.Down);
                break;
            case "buttonLeft":
                result = _drawningWarship.MoveTransport(DirectionType.Left);
                break;
            case "buttonRight":
                result = _drawningWarship.MoveTransport(DirectionType.Right);
                break;
        }

        if (result)
        {
            Draw();
        }
    }

    /// <summary>
    /// Обработка нажатия кнопки "Шаг"
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonStrategyStep_Click(object sender, EventArgs e)
    {
        if (_drawningWarship == null)
        {
            return;
        }

        if (comboBoxStrategy.Enabled)
        {
            _strategy = comboBoxStrategy.SelectedIndex switch
            {
                0 => new MoveToCenter(),
                1 => new MoveToBorder(),
                _ => null,
            };
            if(_strategy == null)
            {
                return;
            }
            _strategy.SetData(new MoveableWarship(_drawningWarship), pictureBoxAircraftCarrier.Width, pictureBoxAircraftCarrier.Height);
        }

        if(_strategy == null)
        {
            return;
        }

        comboBoxStrategy.Enabled = false;
        _strategy.MakeStep();
        Draw();

        if(_strategy.GetStatus() == StrategyStatus.Finish)
        {
            comboBoxStrategy.Enabled = true;
            _strategy = null;
        }
    }
}
